%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% extract_features: The ten most significative features are extracted from
% different databases
%
% Input:
%       - file: FASTA file with the sequences to align
% Output:
%       - features: Vector with the 10 calculated features for each method
%
% Francisco Ortuo 
% Department of Computer Architecture and Computer Technology
% University of Granada
% April 2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [features]=extract_features(file);

% Read the FASTA file with the sequences
try
	sequences = fastaread(file);
catch ME
	disp('The FASTA file cannot be read:');
    disp(ME.message);
end

% Calculate features
features = zeros(1,10);

% 1) Number of sequences
features(1) = length(sequences);

% For each sequence in the input.
for i=1:features(1)
    % Obtain sequence
    secuencia = sequences(i).Sequence;
    
    % Check Uniprot URL with Uniprot nomenclature
    searchurl = ['http://www.uniprot.org/uniprot/' sequences(i).Header '.txt'];
    [s, status] = urlread(searchurl);
    
    % Check Uniprot URL with PDB nomenclature  
    if(status==0)
        
        % Find the Uniprot accession from PDB identifier
        if(length(sequences(i).Header)==4)
            searchurl = ['http://www.bioinf.org.uk/cgi-bin/pdbsws/query.pl?plain=1&qtype=pdb&id=' sequences(i).Header];
        else
            searchurl = ['http://www.bioinf.org.uk/cgi-bin/pdbsws/query.pl?plain=1&qtype=pdb&id=' sequences(i).Header(1:4) '&chain=' sequences(i).Header(6)];
        end
        [s, status] = urlread(searchurl);
        
        % Retrieve Uniprot accession
        accPos= regexp(s, 'AC:') + 4;
        UniprotAcc = s(accPos:accPos+5);
        
        % Check Uniprot URL with Uniprot accession
        searchurl = ['http://www.uniprot.org/uniprot/' UniprotAcc '.txt'];
        [s, status] = urlread(searchurl);
    end
   
   
    % 2) Averaged length of sequences
    features(2) = features(2) + length(secuencia) / features(1);   
    
    % 4) AAs in beta-strand structure 
    if(status)
        t=regexp(s,'\nFT\s+STRAND\s+\d+\s+\d+','match');
        t=cell2mat(t);
        if(~isempty(t))
            [a,b,start,final]=strread(t, '%s %s %d %d\n');
            features(4) = features(4) + sum(final-start+1) / (length(secuencia)*features(1));
        end
    end
    
    % 5) Pfam domains
    if(status)
        t=regexp(s,'\nDR\s+Pfam;','match');
        features(5) = features(5) + length(t) / features(1);    
    end    
             
    % 6) Number of GO terms
    if(status)
        t=regexp(s,'\nDR\s+GO;','match');
        features(6) = features(6) + length(t) / features(1);            
    end
    
    % 7) Number of Polar AAs [GAPVLIM]
    features(7) = features(7) + sum((secuencia=='G'|secuencia=='A'|secuencia=='P'|secuencia=='V'|secuencia=='L'|secuencia=='I'|secuencia=='M')) / (length(secuencia)*features(1)); 
       
    % 8) Number of Basic AAs [KRH]
    features(8) = features(8) + sum((secuencia=='K'|secuencia=='R'|secuencia=='H')) / (length(secuencia)*features(1));
    
    % 9) Number of Acid AAs [DE]
    features(9) = features(9) + sum((secuencia=='D'|secuencia=='E')) / (length(secuencia)*features(1)); 

end

% 3) Variance length
for i=1:features(1)
    features(3) = features(3) + (features(2) - length(sequences(i).Sequence))^2 / features(1);
end

% 10) MSA methods
features = repmat(features,10,1); % Replicate features for each method
features(:,10) = 1:10;

% ----------------
% NORMALIZATION
% ----------------

% Read used features from BAliBASE to normalize
load MSA_features.txt;

% Normalize each feature
features(:,1) = (features(:,1)-min(MSA_features(:,2)))./(max(MSA_features(:,2))-min(MSA_features(:,2)));
features(:,2) = (features(:,2)-min(MSA_features(:,3)))./(max(MSA_features(:,3))-min(MSA_features(:,3)));
features(:,3) = (features(:,3)-min(MSA_features(:,4)))./(max(MSA_features(:,4))-min(MSA_features(:,4)));
features(:,4) = (features(:,4)-min(MSA_features(:,6)))./(max(MSA_features(:,6))-min(MSA_features(:,6)));
features(:,5) = (features(:,5)-min(MSA_features(:,7)))./(max(MSA_features(:,7))-min(MSA_features(:,7)));
features(:,6) = (features(:,6)-min(MSA_features(:,10)))./(max(MSA_features(:,10))-min(MSA_features(:,10)));
features(:,7) = (features(:,7)-min(MSA_features(:,18)))./(max(MSA_features(:,18))-min(MSA_features(:,18)));
features(:,8) = (features(:,8)-min(MSA_features(:,20)))./(max(MSA_features(:,20))-min(MSA_features(:,20)));
features(:,9) = (features(:,9)-min(MSA_features(:,22)))./(max(MSA_features(:,22))-min(MSA_features(:,22)));
features(:,10) = (features(:,10)-min(MSA_features(:,23)))./(max(MSA_features(:,23))-min(MSA_features(:,23)));

